@@ -1,6 +1,6 @@
 """PyVogen：开源开源歌声合成引擎Vogen的python实现"""
 
-__version__='0.0.5'
+__version__='0.0.7'
 
 import copy
 import json
@@ -11,7 +11,7 @@ import pypinyin
 import more_itertools
 from vogen import config
 from typing import List,Tuple,Dict,Union,Optional,Any
-from collections.abc import Callable
+from collections.abc import Callable,Iterator
 
 #导入numpy（可选）
 try:
@@ -88,8 +88,11 @@ class VogUtt():
         return VogUtt(singerId=self.singerId,romScheme=self.romScheme,notes=self.notes.copy()+other.notes.copy())
 
     def __radd__(self,other):
-        #为适配sum，规定：其他类型+VogUtt返回原VogUtt的副本
-        return copy.deepcopy(self)
+        #为适配sum，规定：0+VogUtt返回原VogUtt的副本
+        if(type(other)==int and other==0):
+            return copy.deepcopy(self)
+        else:
+            return NotImplemented
 
     def __str__(self)->str:
         return " VogUtt {} {} \n".format(self.singerId,self.romScheme)+"".join(str(n) for n in self.notes)
@@ -164,7 +167,7 @@ class VogUtt():
         if(self.romScheme=="man"):#普通话
             pinyins=pypinyin.lazy_pinyin(hanzis)
         elif(self.romScheme.startswith("yue")):#粤语
-            pinyins=(i[:,-1] for i in jyutping.get(note))
+            pinyins=(i[:-1] for i in jyutping.get(hanzis))
         for (note,pinyin) in zip(hanzinotes,pinyins):
             note.rom=pinyin
         return self
@@ -192,6 +195,46 @@ class VogUtt():
                 del self.notes[length-i-1]
         return self
 
+    def getlyric(self)->List[str]:
+        """
+        获取乐句中所有音符的歌词（字符串列表）
+        """
+        return [note.lyric for note in self.notes]
+
+    def getrom(self)->List[str]:
+        """
+        获取乐句中所有音符的拼音（字符串列表）
+        """
+        return [note.rom for note in self.notes]
+
+    def setlyric(self,lyrics:Iterator[str]):
+        """
+        批量灌入歌词，并自动转换为拼音
+        lyrics：要输入的歌词，可以是字符串，也可以是字符串列表
+        """
+        for (note,lyric) in zip(self.notes,lyrics):
+            note.lyric=lyric
+        return self.lyrictorom()
+
+    def transpose(self,value:int):
+        """
+        移调
+        """
+        for note in self.notes:
+            note.pitch+=value
+        return self
+
+    def copy(self):
+        """
+        将乐句复制到剪贴板，可粘贴到vogen编辑器
+        """
+        import pyperclip
+        f=copy.deepcopy(self).sort()
+        offset=f[0].on
+        for note in f:
+            note.on-=offset
+        pyperclip.copy(json.dumps({"utts":[f.dump()]}))
+
 def music21StreamToVogUtt(st)->VogUtt:
     """
     将music21 stream对象转为vogen utt对象
@@ -223,7 +266,7 @@ def utaupyUstToVogUtt(u)->VogUtt:
         time+=notelen
     return VogUtt(notes=vognotes)
 
-def midoMidiTrackToVogUtt(mt)->VogUtt:
+def midoMidiTrackToVogUtt(mt,ticks_per_beat:int=480)->VogUtt:
     """
     将mido miditrack对象转为vogen utt对象
     """
@@ -240,8 +283,8 @@ def midoMidiTrackToVogUtt(mt)->VogUtt:
             #从键位-音符字典中找到音符，并弹出
             if(signal.note in note):
                 n=note.pop(signal.note)
-                vognotes.append(VogNote(on=n[1],
-                            dur=tick-n[1],
+                vognotes.append(VogNote(on=n[1]*480//ticks_per_beat,
+                            dur=(tick-n[1])*480//ticks_per_beat,
                             pitch=signal.note,
                             lyric=n[0],
                             rom=n[0]))
@@ -298,8 +341,11 @@ class VogFile():
         return result
 
     def __radd__(self,other):
-        #为适配sum，规定：其他类型+VogUtt返回原VogUtt的副本
-        return copy.deepcopy(self)
+        #为适配sum，规定：0+VogUtt返回原VogUtt的副本
+        if(type(other)==int and other==0):
+            return copy.deepcopy(self)
+        else:
+            return NotImplemented
 
     def __str__(self):
         return "VogFile {} {}\n".format(self.timeSig0,self.bpm0)+"".join(str(utt) for utt in self.utts)
@@ -388,7 +434,7 @@ class VogFile():
         """
         for utt in self.utts:
             utt.romScheme=romScheme
-        return self
+        return self.lyrictorom()
 
     def lyrictorom(self):
         """
@@ -431,6 +477,55 @@ class VogFile():
     def toVogFile(self):
         return copy.deepcopy(self)
 
+    def notes(self):
+        """
+        工程对象中的音符迭代器
+        注意：不是按音符的时间顺序，而是按乐句在工程中的顺序和音符在乐句中的顺序
+        只有在乐句、音符按时间顺序排列（可用VogFile.sort函数排序），且没有乐句重叠时，该迭代器才按时间顺序
+        """
+        for utt in self:
+            yield from utt
+    
+    def getlyric(self)->List[str]:
+        """
+        获取工程中所有音符的歌词（字符串列表）
+        """
+        return [note.lyric for note in self.notes()]
+    
+    def getrom(self)->List[str]:
+        """
+        获取工程中所有音符的拼音（字符串列表）
+        """
+        return [note.rom for note in self.notes()]
+    
+    def setlyric(self,lyrics:Iterator[str]):
+        """
+        批量灌入歌词，并自动转换为拼音
+        lyrics：要输入的歌词，可以是字符串，也可以是字符串列表
+        """
+        for (note,lyric) in zip(self.notes(),lyrics):
+            note.lyric=lyric
+        return self.lyrictorom()   
+
+    def transpose(self,value:int):
+        """
+        移调
+        """
+        for utt in self.utts:
+            utt.transpose(value)
+        return self
+
+    def copy(self):
+        """
+        将工程复制到剪贴板，可粘贴到vogen编辑器
+        """
+        import pyperclip
+        f=copy.deepcopy(self).sort()
+        offset=f[0][0].on
+        for note in f.notes():
+            note.on-=offset
+        pyperclip.copy(json.dumps(f.dump()))
+
 def music21StreamToVogFile(st)->VogFile:
     """
     将music21 stream对象转为vogen工程对象
@@ -462,18 +557,27 @@ def utaupyUstToVogFile(u)->VogFile:
     """
     return VogFile(utts=[utaupyUstToVogUtt(u)],bpm0=float(u.tempo)).autosplit()
 
-def midoMidiTrackToVogFile(mt)->VogFile:
-    vu=midoMidiTrackToVogUtt(mt)
+def getTempoFromMidoMidiFile(mf)->float:
+    """从mido.MidiFile获取曲速"""
+    import mido
+    for track in mf.tracks:
+        for msg in track:
+            if(hasattr(msg,"tempo")):
+                return mido.tempo2bpm(msg.tempo)
+    return 120.0
+
+def midoMidiTrackToVogFile(mt,ticks_per_beat:int=480)->VogFile:
+    vu=midoMidiTrackToVogUtt(mt,ticks_per_beat)
     if(len(vu)>0):
         utts=[vu]
     else:
         utts=[]
     return VogFile(utts=utts).autosplit()
-    #TODO:tempo
 
 def midoMidiFileToVogFile(mf)->VogFile:
-    return sum([midoMidiTrackToVogFile(tr) for tr in mf.tracks]).sort()
-    #TODO:tempo
+    result=sum([midoMidiTrackToVogFile(tr,mf.ticks_per_beat) for tr in mf.tracks]).sort()
+    result.tempo=getTempoFromMidoMidiFile(mf)
+    return result
 
 def toVogFile(a)->VogFile:
     """
@@ -566,6 +670,13 @@ def loadfile(filename:str)->VogFile:
             }
     return fileparsers[filetype](filename)
 
+def paste()->VogFile:
+    """
+    获取剪贴板中的工程片段
+    """
+    import pyperclip
+    return parsefile(json.loads(pyperclip.paste()))
+
 #关于f0格式
 #np.fromfile("utt-0.f0",dtype=np.int)
 #以八度为单位，八度之间等差，差2**23
